;;; -*- Mode: lisp -*-

(defpackage "SEQUENCE-SYSTEM" (:use "CL" "ASDF"))
(in-package "SEQUENCE-SYSTEM")

;;; we include some of these source files directly into documents, so
;;; every line counts, including boilerplate IN-PACKAGE lines.
(defpackage "SCRATCH"
  (:use "CL" "SB-MOP"))

(defclass sequence-file (cl-source-file) ())

(defmethod perform :around ((o compile-op) (s sequence-file))
  (let ((*package* (find-package "SCRATCH")))
    (call-next-method)))
(defmethod perform :around ((o load-op) (s sequence-file))
  (let ((*package* (find-package "SCRATCH")))
    (call-next-method)))
(defmethod perform :around ((o load-source-op) (s sequence-file))
  (let ((*package* (find-package "SCRATCH")))
    (call-next-method)))

(defsystem sequence
  :default-component-class sequence-file
  :components
  (#+nil (:cl-source-file "sequence")
   ;; this one's the tricky one; it's folded into sbcl's sequence
   ;; file, but I need it for the paper.
   #+nil (:file "sequence-loop" :depends-on ("sequence"))
   (:file "queue-classes" #+nil #+nil :depends-on ("sequence"))
   (:file "queue-sequence" :depends-on (#+nil "sequence" "queue-classes"))
   (:file "queue-reader-macro" :depends-on ("queue-classes"))
   (:file "undoable-mixin" #+nil #+nil :depends-on ("sequence"))
   (:file "undoable-mixin-other" :depends-on ("undoable-mixin"))))

;;; Since we've been lazy and not defined a load-form for queues, we
;;; define SEQUENCE-TEST-FILE and ensure that they're always loaded as
;;; source.
(defclass sequence-test-file (cl-source-file) ())

(defmethod perform ((o compile-op) (s sequence-test-file))
  nil)
(defmethod output-files ((o compile-op) (s sequence-test-file))
  nil)
(defmethod perform ((o load-op) (s sequence-test-file))
  (perform (make-instance 'load-source-op) s))
(defmethod input-files ((o load-op) (s sequence-test-file))
  (list (component-pathname s)))

(defsystem sequence-tests
  :depends-on (sb-rt sequence)
  :components
  ((:module "tests"
	    :default-component-class sequence-test-file
            :components ((:file "package")

			 (:file "protocol" :depends-on ("package"))
			 (:file "iteration" :depends-on ("package"))

			 (:file "count" :depends-on ("package"))
			 (:file "find" :depends-on ("package"))
			 (:file "position" :depends-on ("package"))
			 (:file "subseq" :depends-on ("package"))
			 (:file "copy-seq" :depends-on ("package"))
                         (:file "fill" :depends-on ("package"))
			 (:file "substitute" :depends-on ("package"))
                         (:file "replace" :depends-on ("package"))
			 (:file "reverse" :depends-on ("package"))
                         (:file "reduce" :depends-on ("package"))
                         (:file "mismatch" :depends-on ("package"))
                         (:file "search" :depends-on ("package"))
                         (:file "remove" :depends-on ("package"))
                         (:file "remove-duplicates" :depends-on ("package"))
                         (:file "sort" :depends-on ("package"))))))
