(in-package "SEQUENCE-TESTS")

(deftest find.queue.1
    (find 'c #[a b c d e c a])
  c)

(deftest find.queue.2
    (find 'c #[a b c d e c a] :from-end t)
  c)

(deftest find.queue.3
    (loop for i from 0 to 7 collect
	  (find 'c #[a b c d e c a] :start i))
  (c c c c c c nil nil))

(deftest find.queue.4
    (loop for i from 0 to 7 collect
	  (find 'c #[a b c d e c a] :start i :end nil))
  (c c c c c c nil nil))

(deftest find.queue.5
    (loop for i from 7 downto 0 collect
	  (find 'c #[a b c d e c a] :end i))
  (c c c c c nil nil nil))

(deftest find.queue.6
    (loop for i from 0 to 7 collect
	  (find 'c #[a b c d e c a] :start i :from-end t))
  (c c c c c c nil nil))

(deftest find.queue.7
    (loop for i from 0 to 7 collect
	  (find 'c #[a b c d e c a] :start i :end nil :from-end t))
  (c c c c c c nil nil))

(deftest find.queue.8
    (loop for i from 7 downto 0 collect
	  (find 'c #[a b c d e c a] :end i :from-end t))
  (c c c c c nil nil nil))

(deftest find.queue.9
    (loop for i from 0 to 6 collect
	  (loop for j from (1+ i) to 7
		collect
		(find 'c #[a b c d e c a] :start i :end j)))
  ((nil nil c c c c c) (nil c c c c c) (c c c c c)
   (nil nil c c) (nil c c) (c c) (nil)))

(deftest find.queue.10
    (loop for i from 0 to 6 collect
	  (loop for j from (1+ i) to 7
		collect
		(find 'c #[a b c d e c a] :start i :end j :from-end t)))
  ((nil nil c c c c c) (nil c c c c c) (c c c c c)
   (nil nil c c) (nil c c) (c c) (nil)))

(deftest find.queue.11
    (find 5 #[1 2 3 4 5 6 4 8] :key #'1+)
  4)

(deftest find.queue.12
    (find 5 #[1 2 3 4 5 6 4 8] :key '1+)
  4)

(deftest find.queue.13
    (find 5 #[1 2 3 4 5 6 4 8] :key #'1+ :from-end t)
  4)

(deftest find.queue.14
    (find 'a #[a a b a c e d a f a] :test (complement #'eql))
  b)

(deftest find.queue.15
    (find 'a #[a a b a c e d a f a] 
	  :test (complement #'eql) :from-end t)
  f)

(deftest find.queue.16
    (find 'a #[a a b a c e d a f a] :test-not #'eql)
  b)

(deftest find.queue.17
    (find 'a #[a a b a c e d a f a] :test-not 'eql :from-end t)
  f)

(deftest find.queue.18
    (find 'a #[a a b a c e d a f a] :test-not 'eql)
  b)

(deftest find.queue.19
    (find 'a #[a a b a c e d a f a] :test-not #'eql :from-end t)
  f)

(deftest find.queue.20
    (find 'a #[a a b a c e d a f a] :test-not #'eql)
  b)

(deftest find.queue.21
    (find 'a #[a a b a c e d a f a] :test #'eql :start 2)
  a)

(deftest find.queue.22
    (find 'a #[a a b a c e d a f a] :test #'eql :start 2 :end nil)
  a)

(deftest find.queue.23
    (find 'a #[a a b a c e d a f a] :test-not #'eql :start 0 :end 5)
  b)

(deftest find.queue.24
    (find 'a #[a a b a c e d a f a] :test-not #'eql 
	  :start 0 :end 5 :from-end t)
  c)

(deftest find.queue.25
    (find "ab" #["a" #(#\b #\a) #(#\a #\b #\c)
	         #(#\a #\b) #(#\d #\e) f] :test #'equalp)
  #(#\a #\b))

(deftest find.queue.26
    (find 'a #[(c) (b a) (a b c) (a b) (d e) f] :key #'car)
  (a b c))

(deftest find.queue.27
    (find 'a #[(c) (b a) (a b c) (z) (a b) (d e) f] :key #'car :start 3)
  (a b))

(deftest find.queue.28
    (find 'a #[(c) (b a) (a b c) (z) (a b) (d e) (f)]
	  :key #'car :start 2 :from-end t)
  (a b))

(deftest find.queue.29
    (find 10 #[1 2 3 8 20 3 1 21 3] :test #'<)
  20)

(deftest find.queue.30
    (find 10 #[1 2 3 8 20 3 1 21 3] :test-not #'>=)
  20)

(deftest find.queue.allow-other-keys.1
    (find 0 #[1 2 3 4 5] :key #'(lambda (x) (mod x 2))
	  :bad t :allow-other-keys t)
  2)

(deftest find.queue.allow-other-keys.2
    (find 0 #[1 2 3 4 5] :key #'(lambda (x) (mod x 2))
	  :allow-other-keys t :also-bad t)
  2)

;;; The leftmost of two :allow-other-keys arguments is the one that  matters.
(deftest find.queue.allow-other-keys.3
    (find 0 '#[1 2 3 4 5] :key #'(lambda (x) (mod x 2))
	  :allow-other-keys t :allow-other-keys nil :bad t)
  2)

(deftest find.queue.keywords.4
    (find 2 #[1 2 3 4 5] :key #'identity :key #'1+)
  2)

(deftest find.queue.allow-other-keys.5
    (find 'b #[nil a b c nil] :allow-other-keys nil)
  b)

(deftest find-if.queue.1
    (find-if #'identity #[])
  nil)

(deftest find-if.queue.2
    (find-if #'identity #[a])
  a)

(deftest find-if.queue.2a
    (find-if 'identity #[a])
  a)

(deftest find-if.queue.3
    (find-if #'evenp #[1 2 4 8 3 1 6 7])
  2)

(deftest find-if.queue.4
    (find-if #'evenp #[1 2 4 8 3 1 6 7] :from-end t)
  6)

(deftest find-if.queue.5
    (loop for i from 0 to 7 collect
	  (find-if #'evenp #[1 2 4 8 3 1 6 7] :start i))
  (2 2 4 8 6 6 6 nil))

(deftest find-if.queue.6
    (loop for i from 0 to 7 collect
	  (find-if #'evenp #[1 2 4 8 3 1 6 7] :start i :end nil))
  (2 2 4 8 6 6 6 nil))

(deftest find-if.queue.7
    (loop for i from 0 to 7 collect
	  (find-if #'evenp #[1 2 4 8 3 1 6 7] :start i :from-end t))
  (6 6 6 6 6 6 6 nil))

(deftest find-if.queue.8
    (loop for i from 0 to 7 collect
	  (find-if #'evenp #[1 2 4 8 3 1 6 7] :start i :end nil :from-end t))
  (6 6 6 6 6 6 6 nil))

(deftest find-if.queue.9
    (loop for i from 0 to 8 collect
	  (find-if #'evenp #[1 2 4 8 3 1 6 7] :end i))
  (nil nil 2 2 2 2 2 2 2))

(deftest find-if.queue.10
    (loop for i from 0 to 8 collect
	  (find-if #'evenp #[1 2 4 8 3 1 6 7] :end i :from-end t))
  (nil nil 2 4 8 8 8 6 6))

(deftest find-if.queue.11
    (loop for j from 0 to 7
	  collect
	  (loop for i from (1+ j) to 8 collect
		(find-if #'evenp #[1 2 4 8 3 1 6 7] :start j :end i)))
  ((nil 2 2 2 2 2 2 2) (2 2 2 2 2 2 2) (4 4 4 4 4 4)
   (8 8 8 8 8) (nil nil 6 6) (nil 6 6) (6 6) (nil)))

(deftest find-if.queue.12
    (loop for j from 0 to 7
	  collect
	  (loop for i from (1+ j) to 8 collect
		(find-if #'evenp #[1 2 4 8 3 1 6 7] :start j :end i
			 :from-end t)))
  ((nil 2 4 8 8 8 6 6) (2 4 8 8 8 6 6) (4 8 8 8 6 6)
   (8 8 8 6 6) (nil nil 6 6) (nil 6 6) (6 6) (nil)))

(deftest find-if.queue.13
    (loop for i from 0 to 6
	  collect
	  (find-if #'evenp #[1 6 11 32 45 71 100] :key #'1+ :start i))
  (1 11 11 45 45 71 nil))

(deftest find-if.queue.14
    (loop for i from 0 to 6
	  collect
	  (find-if #'evenp #[1 6 11 32 45 71 100] 
		   :key '1+ :start i :from-end t))
  (71 71 71 71 71 71 nil))

(deftest find-if.queue.15
    (loop for i from 0 to 7
	  collect
	  (find-if #'evenp #[1 6 11 32 45 71 100] :key #'1+ :end i))
  (nil 1 1 1 1 1 1 1))

(deftest find-if.queue.16
    (loop for i from 0 to 7
	  collect
	  (find-if #'evenp #[1 6 11 32 45 71 100] :key '1+ :end i :from-end t))
  (nil 1 1 11 11 45 71 71))

(deftest find-if.queue.17
    (loop for j from 0 to 7
	  collect
	  (loop for i from (1+ j) to 8 collect
		(find-if #'oddp #[1 2 4 8 3 1 6 7] :start j :end i :key #'1-)))
  ((nil 2 2 2 2 2 2 2) (2 2 2 2 2 2 2) (4 4 4 4 4 4)
   (8 8 8 8 8) (nil nil 6 6) (nil 6 6) (6 6) (nil)))

(deftest find-if.queue.18
    (loop for j from 0 to 7
	  collect
	  (loop for i from (1+ j) to 8 collect
		(find-if #'oddp #[1 2 4 8 3 1 6 7] :start j :end i
			 :from-end t :key #'1+)))
  ((nil 2 4 8 8 8 6 6) (2 4 8 8 8 6 6) (4 8 8 8 6 6)
   (8 8 8 6 6) (nil nil 6 6) (nil 6 6) (6 6) (nil)))

(deftest find-if.queue.allow-other-keys.1
    (find-if #'evenp #[1 2 3 4 5] :bad t :allow-other-keys t)
  2)

(deftest find-if.queue.allow-other-keys.2
    (find-if #'evenp #[1 2 3 4 5] :allow-other-keys t :also-bad t)
  2)

(deftest find-if.queue.allow-other-keys.3
    (find-if #'evenp #[1 2 3 4 5]
	     :allow-other-keys t :allow-other-keys nil :bad t)
  2)

(deftest find-if.queue.keywords.4
    (find-if #'evenp #[1 2 3 4 5] :key #'identity :key #'1+)
  2)

(deftest find-if.queue.allow-other-keys.5
    (find-if #'identity #[nil a b c nil] :allow-other-keys nil)
  a)

(deftest find-if.queue.order.1
    (let ((i 0) x y)
      (values
       (find-if (progn (setf x (incf i)) #'identity)
		(progn (setf y (incf i)) #[nil nil nil a nil nil]))
       i x y))
  a 2 1 2)

(deftest find-if.queue.order.2
  (let ((i 0) a b c d e f)
    (values
     (find-if (progn (setf a (incf i)) #'null)
	      (progn (setf b (incf i)) #[nil nil nil a nil nil])
	      :start (progn (setf c (incf i)) 1)
	      :end   (progn (setf d (incf i)) 4)
	      :from-end (setf e (incf i))
	      :key   (progn (setf f (incf i)) #'null))
     i a b c d e f))
  a 6 1 2 3 4 5 6)


(deftest find-if.order.3
  (let ((i 0) a b c d e f)
    (values
     (find-if (progn (setf a (incf i)) #'null)
	      (progn (setf b (incf i)) #[nil nil nil a nil nil])
	      :key   (progn (setf c (incf i)) #'null)
	      :from-end (setf d (incf i))
	      :end   (progn (setf e (incf i)) 4)
	      :start (progn (setf f (incf i)) 1))
     i a b c d e f))
  a 6 1 2 3 4 5 6)

(deftest find-if-not.queue.1
    (find-if-not #'identity #[])
  nil)

(deftest find-if-not.queue.2
    (find-if-not #'null #[a])
  a)

(deftest find-if-not.queue.2a
    (find-if-not 'null #[a])
  a)

(deftest find-if-not.queue.3
    (find-if-not #'oddp #[1 2 4 8 3 1 6 7])
  2)

(deftest find-if-not.queue.4
    (find-if-not #'oddp #[1 2 4 8 3 1 6 7] :from-end t)
  6)

(deftest find-if-not.queue.5
    (loop for i from 0 to 7 collect
	  (find-if-not #'oddp #[1 2 4 8 3 1 6 7] :start i))
  (2 2 4 8 6 6 6 nil))

(deftest find-if-not.queue.6
    (loop for i from 0 to 7 collect
	  (find-if-not #'oddp #[1 2 4 8 3 1 6 7] :start i :end nil))
  (2 2 4 8 6 6 6 nil))

(deftest find-if-not.queue.7
    (loop for i from 0 to 7 collect
	  (find-if-not #'oddp #[1 2 4 8 3 1 6 7] :start i :from-end t))
  (6 6 6 6 6 6 6 nil))

(deftest find-if-not.queue.8
    (loop for i from 0 to 7 collect
	  (find-if-not #'oddp #[1 2 4 8 3 1 6 7] 
		       :start i :end nil :from-end t))
  (6 6 6 6 6 6 6 nil))

(deftest find-if-not.queue.9
    (loop for i from 0 to 8 collect
	  (find-if-not #'oddp #[1 2 4 8 3 1 6 7] :end i))
  (nil nil 2 2 2 2 2 2 2))

(deftest find-if-not.queue.10
    (loop for i from 0 to 8 collect
	  (find-if-not #'oddp #[1 2 4 8 3 1 6 7] :end i :from-end t))
  (nil nil 2 4 8 8 8 6 6))

(deftest find-if-not.queue.11
    (loop for j from 0 to 7
	  collect
	  (loop for i from (1+ j) to 8 collect
		(find-if-not #'oddp #[1 2 4 8 3 1 6 7] :start j :end i)))
  ((nil 2 2 2 2 2 2 2) (2 2 2 2 2 2 2) (4 4 4 4 4 4)
   (8 8 8 8 8) (nil nil 6 6) (nil 6 6) (6 6) (nil)))

(deftest find-if-not.queue.12
    (loop for j from 0 to 7
	  collect
	  (loop for i from (1+ j) to 8 collect
		(find-if-not #'oddp #[1 2 4 8 3 1 6 7] :start j :end i
			     :from-end t)))
  ((nil 2 4 8 8 8 6 6) (2 4 8 8 8 6 6) (4 8 8 8 6 6)
   (8 8 8 6 6) (nil nil 6 6) (nil 6 6) (6 6) (nil)))

(deftest find-if-not.queue.13
    (loop for i from 0 to 6
	  collect
	  (find-if-not #'oddp #[1 6 11 32 45 71 100] :key #'1+ :start i))
  (1 11 11 45 45 71 nil))

(deftest find-if-not.queue.14
    (loop for i from 0 to 6
	  collect
	  (find-if-not #'oddp #[1 6 11 32 45 71 100]
		       :key '1+ :start i :from-end t))
  (71 71 71 71 71 71 nil))

(deftest find-if-not.queue.15
    (loop for i from 0 to 7
	  collect
	  (find-if-not #'oddp #[1 6 11 32 45 71 100] :key #'1+ :end i))
  (nil 1 1 1 1 1 1 1))

(deftest find-if-not.queue.16
    (loop for i from 0 to 7
	  collect
	  (find-if-not #'oddp #[1 6 11 32 45 71 100] 
		       :key '1+ :end i :from-end t))
  (nil 1 1 11 11 45 71 71))

(deftest find-if-not.queue.17
    (loop for j from 0 to 7
	  collect
	  (loop for i from (1+ j) to 8 collect
		(find-if-not #'evenp #[1 2 4 8 3 1 6 7] :start j :end i :key #'1-)))
  ((nil 2 2 2 2 2 2 2) (2 2 2 2 2 2 2) (4 4 4 4 4 4)
   (8 8 8 8 8) (nil nil 6 6) (nil 6 6) (6 6) (nil)))

(deftest find-if-not.queue.18
    (loop for j from 0 to 7
	  collect
	  (loop for i from (1+ j) to 8 collect
		(find-if-not #'evenp #[1 2 4 8 3 1 6 7] :start j :end i
			     :from-end t :key #'1+)))
  ((nil 2 4 8 8 8 6 6) (2 4 8 8 8 6 6) (4 8 8 8 6 6)
   (8 8 8 6 6) (nil nil 6 6) (nil 6 6) (6 6) (nil)))

(deftest find-if-not.queue.allow-other-keys.1
    (find-if-not #'oddp #[1 2 3 4 5] :bad t :allow-other-keys t)
  2)

(deftest find-if-not.queue.allow-other-keys.2
    (find-if-not #'oddp #[1 2 3 4 5] :allow-other-keys t :also-bad t)
  2)

;;; The leftmost of two :allow-other-keys arguments is the one that  matters.
(deftest find-if-not.queue.allow-other-keys.3
    (find-if-not #'oddp #[1 2 3 4 5]
		 :allow-other-keys t :allow-other-keys nil :bad t)
  2)

(deftest find-if-not.queue.keywords.4
    (find-if-not #'oddp #[1 2 3 4 5] :key #'identity :key #'1+)
  2)

(deftest find-if-not.queue.allow-other-keys.5
    (find-if-not #'null #[nil a b c nil] :allow-other-keys nil)
  a)

(deftest find-if-not.queue.order.1
    (let ((i 0) x y)
      (values
       (find-if-not (progn (setf x (incf i)) #'null)
		    (progn (setf y (incf i)) #[nil nil nil a nil nil]))
       i x y))
  a 2 1 2)

(deftest find-if-not.queue.order.2
  (let ((i 0) a b c d e f)
    (values
     (find-if-not (progn (setf a (incf i)) #'identity)
		  (progn (setf b (incf i)) #[nil nil nil a nil nil])
		  :start (progn (setf c (incf i)) 1)
		  :end   (progn (setf d (incf i)) 4)
		  :from-end (setf e (incf i))
		  :key   (progn (setf f (incf i)) #'null)
		  )
     i a b c d e f))
  a 6 1 2 3 4 5 6)


(deftest find-if-not.queue.order.3
  (let ((i 0) a b c d e f)
    (values
     (find-if-not (progn (setf a (incf i)) #'identity)
		  (progn (setf b (incf i)) #[nil nil nil a nil nil])
		  :key   (progn (setf c (incf i)) #'null)
		  :from-end (setf d (incf i))
		  :end   (progn (setf e (incf i)) 4)
		  :start (progn (setf f (incf i)) 1)
		  )
     i a b c d e f))
  a 6 1 2 3 4 5 6)
