(in-package "SEQUENCE-TESTS")

(defmacro expand-in-current-env (form &environment env)
  (macroexpand form env))

(deftest dosequence.list.1
    (let ((count 0))
      (dosequence (e '(a b nil d)) (incf count))
      count)
  4)

(deftest dosequence.list.2
    (let ((count 0))
      (dosequence (e '(a b nil d) count) (incf count)))
  4)

(deftest dosequence.list.3
    (let ((count 0))
      (dosequence (x nil count) (incf count)))
  0)

(deftest dosequence.list.4
    (let ((y nil))
      (flet ((%f () (locally (declare (special e)) (push e y))))
        (dosequence (e '(a b c) (reverse y))
          (declare (special e))
          (%f))))
  (a b c))

(deftest dosequence.list.5
    (let ((even nil) (odd nil))
      (dosequence (i '(1 2 3 4 5 6 7 8) (values (reverse even) (reverse odd)))
        (when (evenp i) (go even))
        (push i odd)
        (go done)
        even
        (push i even)
        done))
  (2 4 6 8)
  (1 3 5 7))

(deftest dosequence.list.6
    (let ((i 0) (y nil))
      (declare (special i))
      (flet ((%f () i))
        (dosequence (i '(1 2 3 4))
          (push (%f) y)))
      y)
  (0 0 0 0))

(deftest dosequence.list.7
    (dosequence (x '(a b) (values))))

(deftest dosequence.list.8
    (let ((count 0))
      (dosequence (x '(a b c) (values count count))
        (incf count)))
  3 3)

(deftest dosequence.list.9
    (block nil
      (eql (dosequence (x '(a b c)) (return 1)) 1))
  t)

(deftest dosequence.list.10
    (block nil
      (eql (dosequence (x '(a b c)) (return-from nil 1)) 1))
  t)

(deftest dosequence.list.11
    (block nil (dosequence (x (return 1))) 2)
  2)

(deftest dosequence.list.12
    (block nil (dosequence (x '(a b) (return 1))) 2)
  2)

(deftest dosequence.list.13
    (dosequence (e '(a b c) e))
  nil)

(deftest dosequence.list.14
    (let ((e 1))
      (dosequence (e '(a b c) (setf e 2)))
      e)
  1)

(deftest dosequence.list.15
    (let ((x nil))
      (dosequence (e '(a b c d e f))
        (push e x)
        (when (eq e 'c) (return x))))
  (c b a))

(deftest dosequence.list.16
    (block done
      (let ((x :bad))
        (declare (special x))
        (let ((x :good))
          (dosequence (e (return-from done x))
            (declare (special x))))))
  :good)

(deftest dosequence.list.17
    (let ((x :good))
      (declare (special x))
      (let ((x :bad))
        (dosequence (e nil x)
          (declare (special x)))))
  :good)

(deftest dosequence.list.18
    (let ((result nil))
      (macrolet ((%m (z) z))
        (dosequence (x (expand-in-current-env (%m '(a b c))) result)
          (push x result))))
  (c b a))

(deftest dosequence.list.19
    (let ((result nil))
      (macrolet ((%m (z) z))
        (dosequence (x '(a b c) (expand-in-current-env (%m result)))
          (push x result))))
  (c b a))

(deftest dosequence.vector.1
    (let ((count 0))
      (dosequence (e #(a b nil d)) (incf count))
      count)
  4)

(deftest dosequence.vector.2
    (let ((count 0))
      (dosequence (e #(a b nil d) count) (incf count)))
  4)

(deftest dosequence.vector.3
    (let ((count 0))
      (dosequence (x #() count) (incf count)))
  0)

(deftest dosequence.vector.4
    (let ((y nil))
      (flet ((%f () (locally (declare (special e)) (push e y))))
        (dosequence (e #(a b c) (reverse y))
          (declare (special e))
          (%f))))
  (a b c))

(deftest dosequence.vector.5
    (let ((even nil) (odd nil))
      (dosequence (i #(1 2 3 4 5 6 7 8) (values (reverse even) (reverse odd)))
        (when (evenp i) (go even))
        (push i odd)
        (go done)
        even
        (push i even)
        done))
  (2 4 6 8)
  (1 3 5 7))

(deftest dosequence.vector.6
    (let ((i 0) (y nil))
      (declare (special i))
      (flet ((%f () i))
        (dosequence (i #(1 2 3 4))
          (push (%f) y)))
      y)
  (0 0 0 0))

(deftest dosequence.vector.7
    (dosequence (x #(a b) (values))))

(deftest dosequence.vector.8
    (let ((count 0))
      (dosequence (x #(a b c) (values count count))
        (incf count)))
  3 3)

(deftest dosequence.vector.9
    (block nil
      (eql (dosequence (x #(a b c)) (return 1)) 1))
  t)

(deftest dosequence.vector.10
    (block nil
      (eql (dosequence (x #(a b c)) (return-from nil 1)) 1))
  t)

(deftest dosequence.vector.11
    (block nil (dosequence (x (return 1))) 2)
  2)

(deftest dosequence.vector.12
    (block nil (dosequence (x #(a b) (return 1))) 2)
  2)

(deftest dosequence.vector.13
    (dosequence (e #(a b c) e))
  nil)

(deftest dosequence.vector.14
    (let ((e 1))
      (dosequence (e #(a b c) (setf e 2)))
      e)
  1)

(deftest dosequence.vector.15
    (let ((x nil))
      (dosequence (e #(a b c d e f))
        (push e x)
        (when (eq e 'c) (return x))))
  (c b a))

(deftest dosequence.vector.16
    (block done
      (let ((x :bad))
        (declare (special x))
        (let ((x :good))
          (dosequence (e (return-from done x))
            (declare (special x))))))
  :good)

(deftest dosequence.vector.17
    (let ((x :good))
      (declare (special x))
      (let ((x :bad))
        (dosequence (e #() x)
          (declare (special x)))))
  :good)

(deftest dosequence.vector.18
    (let ((result nil))
      (macrolet ((%m (z) z))
        (dosequence (x (expand-in-current-env (%m #(a b c))) result)
          (push x result))))
  (c b a))

(deftest dosequence.vector.19
    (let ((result nil))
      (macrolet ((%m (z) z))
        (dosequence (x #(a b c) (expand-in-current-env (%m result)))
          (push x result))))
  (c b a))

(deftest dosequence.queue.1
    (let ((count 0))
      (dosequence (e #[a b nil d]) (incf count))
      count)
  4)

(deftest dosequence.queue.2
    (let ((count 0))
      (dosequence (e #[a b nil d] count) (incf count)))
  4)

(deftest dosequence.queue.3
    (let ((count 0))
      (dosequence (x #[] count) (incf count)))
  0)

(deftest dosequence.queue.4
    (let ((y nil))
      (flet ((%f () (locally (declare (special e)) (push e y))))
        (dosequence (e #[a b c] (reverse y))
          (declare (special e))
          (%f))))
  (a b c))

(deftest dosequence.queue.5
    (let ((even nil) (odd nil))
      (dosequence (i #[1 2 3 4 5 6 7 8] (values (reverse even) (reverse odd)))
        (when (evenp i) (go even))
        (push i odd)
        (go done)
        even
        (push i even)
        done))
  (2 4 6 8)
  (1 3 5 7))

(deftest dosequence.queue.6
    (let ((i 0) (y nil))
      (declare (special i))
      (flet ((%f () i))
        (dosequence (i #[1 2 3 4])
          (push (%f) y)))
      y)
  (0 0 0 0))

(deftest dosequence.queue.7
    (dosequence (x #[a b] (values))))

(deftest dosequence.queue.8
    (let ((count 0))
      (dosequence (x #[a b c] (values count count))
        (incf count)))
  3 3)

(deftest dosequence.queue.9
    (block nil
      (eql (dosequence (x #[a b c]) (return 1)) 1))
  t)

(deftest dosequence.queue.10
    (block nil
      (eql (dosequence (x #[a b c]) (return-from nil 1)) 1))
  t)

(deftest dosequence.queue.11
    (block nil (dosequence (x (return 1))) 2)
  2)

(deftest dosequence.queue.12
    (block nil (dosequence (x #[a b] (return 1))) 2)
  2)

(deftest dosequence.queue.13
    (dosequence (e #[a b c] e))
  nil)

(deftest dosequence.queue.14
    (let ((e 1))
      (dosequence (e #[a b c] (setf e 2)))
      e)
  1)

(deftest dosequence.queue.15
    (let ((x nil))
      (dosequence (e #[a b c d e f])
        (push e x)
        (when (eq e 'c) (return x))))
  (c b a))

(deftest dosequence.queue.16
    (block done
      (let ((x :bad))
        (declare (special x))
        (let ((x :good))
          (dosequence (e (return-from done x))
            (declare (special x))))))
  :good)

(deftest dosequence.queue.17
    (let ((x :good))
      (declare (special x))
      (let ((x :bad))
        (dosequence (e #[] x)
          (declare (special x)))))
  :good)

(deftest dosequence.queue.18
    (let ((result nil))
      (macrolet ((%m (z) z))
        (dosequence (x (expand-in-current-env (%m #[a b c])) result)
          (push x result))))
  (c b a))

(deftest dosequence.queue.19
    (let ((result nil))
      (macrolet ((%m (z) z))
        (dosequence (x #[a b c] (expand-in-current-env (%m result)))
          (push x result))))
  (c b a))

(deftest loop.queue.1
    (loop for x being the elements of #[1 2 3] sum x)
  6)

(deftest loop.queue.2
    (loop for x fixnum being the elements of #[1 2 3] sum x)
  6)

(deftest loop.queue.3
    (loop for x of-type fixnum being the elements of #[1 2 3] sum x)
  6)

(deftest loop.queue.4
    (loop for (x y) being the elements of #[(1 t) (2 nil) (3 t)]
	  when y sum x)
  4)

(deftest loop.queue.5
    (loop for (x y) of-type (fixnum (member t nil)) being the elements of #[(1 t) (2 nil) (3 t)]
	  unless y sum x)
  2)
