(in-package "SEQUENCE-TESTS")

(deftest/sequence substitute.queue.1
    (let* ((x #[])
	   (result (substitute 'b 'a x)))
      (and (sequence-equal #[] x) result))
  #[])

(deftest/sequence substitute.queue.2
    (let* ((x #[a b a c])
	   (result (substitute 'b 'a x)))
      (and (sequence-equal #[a b a c] x) result))
  #[b b b c])

(deftest/sequence substitute.queue.3
    (let* ((x #[a b a c])
	   (result (substitute 'b 'a x :count nil)))
      (and (sequence-equal #[a b a c] x) result))
  #[b b b c])

(deftest/sequence substitute.queue.4
    (let* ((x #[a b a c])
	   (result (substitute 'b 'a x :count 2)))
      (and (sequence-equal #[a b a c] x) result))
  #[b b b c])

(deftest/sequence substitute.queue.5
    (let* ((x #[a b a c])
	   (result (substitute 'b 'a x :count 1)))
      (and (sequence-equal #[a b a c] x) result))
  #[b b a c])

(deftest/sequence substitute.queue.6
    (let* ((x #[a b a c])
	   (result (substitute 'b 'a x :count 0)))
      (and (sequence-equal #[a b a c] x) result))
  #[a b a c])

(deftest/sequence substitute.queue.7
    (let* ((x #[a b a c])
	   (result (substitute 'b 'a x :count -1)))
      (and (sequence-equal #[a b a c] x) result))
  #[a b a c])

(deftest/sequence substitute.queue.8
    (let ((x #[])) 
      (substitute 'b 'a x :from-end t))
  #[])

(deftest/sequence substitute.queue.9
    (let* ((x #[a b a c])
	   (result (substitute 'b 'a x :from-end t)))
      (and (sequence-equal #[a b a c] x) result))
  #[b b b c])

(deftest/sequence substitute.queue.10
    (let* ((x #[a b a c])
	   (result (substitute 'b 'a x :from-end t :count nil)))
      (and (sequence-equal #[a b a c] x) result))
  #[b b b c])

(deftest/sequence substitute.queue.11
    (let* ((x #[a b a c])
	   (result (substitute 'b 'a x :count 2 :from-end t)))
      (and (sequence-equal #[a b a c] x) result))
  #[b b b c])

(deftest/sequence substitute.queue.12
    (let* ((x #[a b a c])
	   (result (substitute 'b 'a x :count 1 :from-end t)))
      (and (sequence-equal #[a b a c] x) result))
  #[a b b c])

(deftest/sequence substitute.queue.13
    (let* ((x #[a b a c])
	   (result (substitute 'b 'a x :count 0 :from-end t)))
      (and (sequence-equal #[a b a c] x) result))
  #[a b a c])

(deftest/sequence substitute.queue.14
    (let* ((x #[a b a c])
	   (result (substitute 'b 'a x :count -1 :from-end t)))
      (and (sequence-equal #[a b a c] x) result))
  #[a b a c])

(deftest substitute.queue.15
    (loop for i from 0 to 9 always
	  (loop for j from i to 10 always
		(let* ((orig #[a a a a a a a a a a])
		       (x (copy-seq orig))
		       (y (substitute 'x 'a x :start i :end j)))
		  (and (sequence-equal orig x)
		       (equal (coerce y 'list)
			      (nconc (make-list i :initial-element 'a)
				     (make-list (- j i) :initial-element 'x)
				     (make-list (- 10 j) :initial-element 'a)))))))
  t)

(deftest substitute.queue.16
    (loop for i from 0 to 9 always
	  (loop for j from i to 10 always
		(let* ((orig #[a a a a a a a a a a])
		       (x (copy-seq orig))
		       (y (substitute 'x 'a x :start i :end j :from-end t)))
		  (and (sequence-equal orig x)
		       (equal (coerce y 'list)
			      (nconc (make-list i :initial-element 'a)
				     (make-list (- j i) :initial-element 'x)
				     (make-list (- 10 j) :initial-element 'a)))))))
  t)

(deftest substitute.queue.17
    (loop for i from 0 to 9 always
	  (loop for j from i to 10 always
		(loop for c from 0 to (- j i) always
		      (let* ((orig #[a a a a a a a a a a])
			     (x (copy-seq orig))
			     (y (substitute 'x 'a x :start i :end j :count c)))
			(and (sequence-equal orig x)
			     (equal (coerce y 'list)
				    (nconc (make-list i :initial-element 'a)
					   (make-list c :initial-element 'x)
					   (make-list (- 10 (+ i c)) :initial-element 'a))))))))
  t)

(deftest substitute.queue.18
    (loop for i from 0 to 9 always
	  (loop for j from i to 10 always
		(loop for c from 0 to (- j i) always
		      (let* ((orig #[a a a a a a a a a a])
			     (x (copy-seq orig))
			     (y (substitute 'x 'a x :start i :end j :count c :from-end t)))
			(and (sequence-equal orig x)
			     (equal (coerce y 'list)
				    (nconc (make-list (- j c) :initial-element 'a)
					   (make-list c :initial-element 'x)
					   (make-list (- 10 j) :initial-element 'a))))))))
  t)

(deftest/sequence substitute.queue.19
    (let* ((orig #[1 2 3 4 5 6 7 8 9])
	   (x (copy-seq orig))
	   (result (substitute 'x 5 x :test #'(lambda (a b) (<= (abs (- a b)) 2)))))
      (and (sequence-equal orig x) result))
  #[1 2 x x x x x 8 9])

(deftest/sequence substitute.queue.20
    (let* ((orig #[1 2 3 4 5 6 7 8 9])
	   (x (copy-seq orig))
	   (c -4)
	   (result (substitute 'x 5 x :test #'(lambda (a b) (incf c 2) (= (+ b c) a)))))
      (and (sequence-equal orig x) result))
  #[1 2 x 4 5 6 7 8 9])

(deftest/sequence substitute.queue.21
    (let* ((orig #[1 2 3 4 5 6 7 8 9])
	   (x (copy-seq orig))
	   (c 5)
	   (result (substitute 'x 9 x :test #'(lambda (a b) (incf c -2) (= (+ b c) a))
			       :from-end t)))
      (and (sequence-equal orig x) result))
  #[1 2 3 4 5 6 7 x 9])

(deftest/sequence substitute.queue.22
    (let* ((orig #[1 2 3 4 5 6 7 8 9])
	   (x (copy-seq orig))
	   (c -4)
	   (result (substitute 'x 5 x :test-not #'(lambda (a b) (incf c 2) (/= (+ b c) a)))))
      (and (sequence-equal orig x) result))
  #[1 2 x 4 5 6 7 8 9])


(deftest/sequence substitute.queue.23
    (let* ((orig #[1 2 3 4 5 6 7 8 9])
	   (x (copy-seq orig))
	   (c 5)
	   (result (substitute 'x 9 x :test-not #'(lambda (a b) (incf c -2) (/= (+ b c) a))
			       :from-end t)))
      (and (sequence-equal orig x) result))
  #[1 2 3 4 5 6 7 x 9])

(deftest substitute.queue.24
    (let* ((orig #[(a 1) (b 2) (a 3) (c 4) (d 5) (a 6) (e 7)])
	   (x (copy-seq orig))
	   (result (substitute '(a 10) 'a x :key #'car)))
      (and (sequence-equal orig x) (coerce result 'list)))
  ((a 10) (b 2) (a 10) (c 4) (d 5) (a 10) (e 7)))

(deftest substitute.queue.25
    (let* ((orig #[(a 1) (b 2) (a 3) (c 4) (d 5) (a 6) (e 7)])
	   (x (copy-seq orig))
	   (result (substitute '(a 10) 'a x :key #'car :start 1 :end 5)))
      (and (sequence-equal orig x) (coerce result 'list)))
  ((a 1) (b 2) (a 10) (c 4) (d 5) (a 6) (e 7)))

(deftest substitute.queue.26
    (let* ((orig #[(a 1) (b 2) (a 3) (c 4) (d 5) (a 6) (e 7)])
	   (x (copy-seq orig))
	   (result (substitute '(a 10) 'a x :key #'car :test (complement #'eql))))
      (and (sequence-equal orig x) (coerce result 'list)))
  ((a 1) (a 10) (a 3) (a 10) (a 10) (a 6) (a 10)))

(deftest substitute.queue.27
    (let* ((orig #[(a 1) (b 2) (a 3) (c 4) (d 5) (a 6) (e 7)])
	   (x (copy-seq orig))
	   (result (substitute '(a 10) 'a x :key #'car :test-not #'eql)))
      (and (sequence-equal orig x) (coerce result 'list)))
  ((a 1) (a 10) (a 3) (a 10) (a 10) (a 6) (a 10)))    


(deftest/sequence nsubstitute.queue.1
    (nsubstitute 'b 'a (copy-seq #[]))
  #[])

(deftest/sequence nsubstitute.queue.2
    (let ((x (copy-seq #[a b a c]))) 
      (nsubstitute 'b 'a x) x)
  #[b b b c])

(deftest/sequence nsubstitute.queue.3
    (let ((x (copy-seq #[a b a c]))) 
      (nsubstitute 'b 'a x :count nil))
  #[b b b c])

(deftest/sequence nsubstitute.queue.4
    (let ((x (copy-seq #[a b a c]))) 
      (nsubstitute 'b 'a x :count 2))
  #[b b b c])

(deftest/sequence nsubstitute.queue.5
    (let ((x (copy-seq #[a b a c]))) 
      (nsubstitute 'b 'a x :count 1))
  #[b b a c])

(deftest/sequence nsubstitute.queue.6
    (let ((x (copy-seq #[a b a c]))) 
      (nsubstitute 'b 'a x :count 0))
  #[a b a c])

(deftest/sequence nsubstitute.queue.7
    (let ((x (copy-seq #[a b a c]))) 
      (nsubstitute 'b 'a x :count -1))
  #[a b a c])

(deftest/sequence nsubstitute.queue.8
    (nsubstitute 'b 'a (copy-seq #[]) :from-end t)
  #[])

(deftest/sequence nsubstitute.queue.9
    (let ((x (copy-seq #[a b a c]))) 
      (nsubstitute 'b 'a x :from-end t))
  #[b b b c])

(deftest/sequence nsubstitute.queue.10
    (let ((x (copy-seq #[a b a c]))) 
      (nsubstitute 'b 'a x :from-end t :count nil))
  #[b b b c])

(deftest/sequence nsubstitute.queue.11
    (let ((x (copy-seq #[a b a c]))) 
      (nsubstitute 'b 'a x :count 2 :from-end t))
  #[b b b c])

(deftest/sequence nsubstitute.queue.12
    (let ((x (copy-seq #[a b a c]))) 
      (nsubstitute 'b 'a x :count 1 :from-end t))
  #[a b b c])

(deftest/sequence nsubstitute.queue.13
    (let ((x (copy-seq #[a b a c]))) 
      (nsubstitute 'b 'a x :count 0 :from-end t))
  #[a b a c])

(deftest/sequence nsubstitute.queue.14
    (let ((x (copy-seq #[a b a c]))) 
      (nsubstitute 'b 'a x :count -1 :from-end t))
  #[a b a c])

(deftest nsubstitute.queue.15
    (loop for i from 0 to 9 always
	  (loop for j from i to 10 always
		(let* ((orig #[a a a a a a a a a a])
		       (x (copy-seq orig))
		       (y (nsubstitute 'x 'a x :start i :end j)))
		  (equal (coerce y 'list)
			 (nconc (make-list i :initial-element 'a)
				(make-list (- j i) :initial-element 'x)
				(make-list (- 10 j) :initial-element 'a))))))
  t)

(deftest nsubstitute.queue.16
    (loop for i from 0 to 9 always
	  (loop for j from i to 10 always
		(let* ((orig #[a a a a a a a a a a])
		       (x (copy-seq orig))
		       (y (nsubstitute 'x 'a x :start i :end j :from-end t)))
		  (equal (coerce y 'list)
			 (nconc (make-list i :initial-element 'a)
				(make-list (- j i) :initial-element 'x)
				(make-list (- 10 j) :initial-element 'a))))))
  t)

(deftest nsubstitute.queue.17
    (loop for i from 0 to 9 always
	  (loop for j from i to 10 always
		(loop for c from 0 to (- j i) always
		      (let* ((orig #[a a a a a a a a a a])
			     (x (copy-seq orig))
			     (y (nsubstitute 'x 'a x :start i :end j :count c)))
			(equal (coerce y 'list)
			       (nconc (make-list i :initial-element 'a)
				      (make-list c :initial-element 'x)
				      (make-list (- 10 (+ i c)) :initial-element 'a)))))))
  t)

(deftest nsubstitute.queue.18
    (loop for i from 0 to 9 always
	  (loop for j from i to 10 always
		(loop for c from 0 to (- j i) always
		      (let* ((orig #[a a a a a a a a a a])
			     (x (copy-seq orig))
			     (y (nsubstitute 'x 'a x :start i :end j :count c :from-end t)))
			(equal (coerce y 'list)
			       (nconc (make-list (- j c) :initial-element 'a)
				      (make-list c :initial-element 'x)
				      (make-list (- 10 j) :initial-element 'a)))))))
  t)

(deftest/sequence nsubstitute.queue.19
    (let* ((orig #[1 2 3 4 5 6 7 8 9])
	   (x (copy-seq orig))
	   (result (nsubstitute 'x 5 x :test #'(lambda (a b) (<= (abs (- a b)) 2)))))
      result)
  #[1 2 x x x x x 8 9])

(deftest/sequence nsubstitute.queue.20
    (let* ((orig #[1 2 3 4 5 6 7 8 9])
	   (x (copy-seq orig))
	   (c -4)
	   (result (nsubstitute 'x 5 x :test #'(lambda (a b) (incf c 2) (= (+ b c) a)))))
      result)
  #[1 2 x 4 5 6 7 8 9])

(deftest/sequence nsubstitute.queue.21
    (let* ((orig #[1 2 3 4 5 6 7 8 9])
	   (x (copy-seq orig))
	   (c 5)
	   (result (nsubstitute 'x 9 x :test #'(lambda (a b) (incf c -2) (= (+ b c) a))
				:from-end t)))
      result)
  #[1 2 3 4 5 6 7 x 9])

(deftest/sequence nsubstitute.queue.22
    (let* ((orig #[1 2 3 4 5 6 7 8 9])
	   (x (copy-seq orig))
	   (c -4)
	   (result (nsubstitute 'x 5 x :test-not #'(lambda (a b) (incf c 2) (/= (+ b c) a)))))
      result)
  #[1 2 x 4 5 6 7 8 9])

(deftest/sequence nsubstitute.queue.23
    (let* ((orig #[1 2 3 4 5 6 7 8 9])
	   (x (copy-seq orig))
	   (c 5)
	   (result (nsubstitute 'x 9 x :test-not #'(lambda (a b) (incf c -2) (/= (+ b c) a))
				:from-end t)))
      result)
  #[1 2 3 4 5 6 7 x 9])

(deftest/sequence substitute-if.queue.1
    (let* ((x #[])
	   (result (substitute-if 'b #'identity x)))
      (and (sequence-equal #[] x) result))
  #[])

(deftest/sequence substitute-if.queue.2
    (let* ((x #[a b a c])
	   (result (substitute-if 'b (lambda (y) (eql y 'a)) x :count nil)))
      (and (sequence-equal #[a b a c] x) result))
  #[b b b c])

(deftest/sequence substitute-if.queue.3
    (let* ((x #[a b a c])
	   (result (substitute-if 'b (lambda (y) (eql y 'a)) x :count -1)))
      (and (sequence-equal #[a b a c] x) result))
  #[a b a c])

(deftest/sequence substitute-if.queue.4
    (let ((x #[])) 
      (substitute-if 'b #'identity x :from-end t))
  #[])

(deftest/sequence substitute-if.queue.5
    (let* ((orig #[1 2 3 4 5 6 7 8 9])
	   (x (copy-seq orig))
	   (result (substitute-if 'x #'(lambda (b) (<= (abs (- 5 b)) 2)) x)))
      (and (sequence-equal orig x) result))
  #[1 2 x x x x x 8 9])

(deftest/sequence substitute-if.queue.6
    (let* ((orig #[1 2 3 4 5 6 7 8 9])
	   (x (copy-seq orig))
	   (c -4)
	   (result (substitute-if 'x #'(lambda (b) (incf c 2) (= (+ b c) 5)) x)))
      (and (sequence-equal orig x) result))
  #[1 2 x 4 5 6 7 8 9])

(deftest/sequence nsubstitute-if.queue.1
    (let* ((x (copy-seq #[]))
	   (result (nsubstitute-if 'b #'identity x)))
      (and (sequence-equal #[] x) result))
  #[])

(deftest/sequence nsubstitute-if.queue.2
    (let* ((x (copy-seq #[a b a c]))
	   (result (nsubstitute-if 'b (lambda (y) (eql y 'a)) x :count nil)))
      (and (sequence-equal #[b b b c] x) result))
  #[b b b c])

(deftest/sequence nsubstitute-if.queue.3
    (let* ((x (copy-seq #[a b a c]))
	   (result (nsubstitute-if 'b (lambda (y) (eql y 'a)) x :count -1)))
      (and (sequence-equal #[a b a c] x) result))
  #[a b a c])

(deftest/sequence nsubstitute-if.queue.4
    (let ((x (copy-seq #[])) )
      (nsubstitute-if 'b #'identity x :from-end t))
  #[])

(deftest/sequence nsubstitute-if.queue.5
    (let* ((orig (copy-seq #[1 2 3 4 5 6 7 8 9]))
	   (x (copy-seq orig))
	   (result (nsubstitute-if 'x #'(lambda (b) (<= (abs (- 5 b)) 2)) x)))
      (and (sequence-equal x result) result))
  #[1 2 x x x x x 8 9])

(deftest/sequence nsubstitute-if.queue.6
    (let* ((orig #[1 2 3 4 5 6 7 8 9])
	   (x (copy-seq orig))
	   (c -4)
	   (result (nsubstitute-if 'x #'(lambda (b) (incf c 2) (= (+ b c) 5)) x)))
      (and (sequence-equal x result) result))
  #[1 2 x 4 5 6 7 8 9])

(deftest/sequence substitute-if-not.queue.1
    (let* ((x #[])
	   (result (substitute-if-not 'b #'identity x)))
      (and (sequence-equal #[] x) result))
  #[])

(deftest/sequence substitute-if-not.queue.2
    (let* ((x #[a b a c])
	   (result (substitute-if-not 'b (lambda (y) (eql y 'a)) x :count nil)))
      (and (sequence-equal #[a b a c] x) result))
  #[a b a b])

(deftest/sequence substitute-if-not.queue.3
    (let* ((x #[a b a c])
	   (result (substitute-if-not 'b (lambda (y) (eql y 'a)) x :count -1)))
      (and (sequence-equal #[a b a c] x) result))
  #[a b a c])

(deftest/sequence substitute-if-not.queue.4
    (let ((x #[])) 
      (substitute-if-not 'b #'identity x :from-end t))
  #[])

(deftest/sequence substitute-if-not.queue.5
    (let* ((orig #[1 2 3 4 5 6 7 8 9])
	   (x (copy-seq orig))
	   (result (substitute-if-not 'x #'(lambda (b) (<= (abs (- 5 b)) 2)) x)))
      (and (sequence-equal orig x) result))
  #[x x 3 4 5 6 7 x x])

(deftest/sequence substitute-if-not.queue.6
    (let* ((orig #[1 2 3 4 5 6 7 8 9])
	   (x (copy-seq orig))
	   (c -4)
	   (result (substitute-if-not 'x #'(lambda (b) (incf c 2) (= (+ b c) 5)) x)))
      (and (sequence-equal orig x) result))
  #[x x 3 x x x x x x])

(deftest/sequence nsubstitute-if-not.queue.1
    (let* ((x (copy-seq #[]))
	   (result (nsubstitute-if-not 'b #'identity x)))
      (and (sequence-equal #[] x) result))
  #[])

(deftest/sequence nsubstitute-if-not.queue.2
    (let* ((x (copy-seq #[a b a c]))
	   (result (nsubstitute-if-not 'b (lambda (y) (eql y 'a)) x :count nil)))
      (and (sequence-equal x result) result))
  #[a b a b])

(deftest/sequence nsubstitute-if-not.queue.3
    (let* ((x (copy-seq #[a b a c]))
	   (result (nsubstitute-if-not 'b (lambda (y) (eql y 'a)) x :count -1)))
      (and (sequence-equal x result) result))
  #[a b a c])

(deftest/sequence nsubstitute-if-not.queue.4
    (let ((x (copy-seq #[])) )
      (nsubstitute-if-not 'b #'identity x :from-end t))
  #[])

(deftest/sequence nsubstitute-if-not.queue.5
    (let* ((orig (copy-seq #[1 2 3 4 5 6 7 8 9]))
	   (x (copy-seq orig))
	   (result (nsubstitute-if-not 'x #'(lambda (b) (<= (abs (- 5 b)) 2)) x)))
      (and (sequence-equal x result) result))
  #[x x 3 4 5 6 7 x x])

(deftest/sequence nsubstitute-if-not.queue.6
    (let* ((orig #[1 2 3 4 5 6 7 8 9])
	   (x (copy-seq orig))
	   (c -4)
	   (result (nsubstitute-if-not 'x #'(lambda (b) (incf c 2) (= (+ b c) 5)) x)))
      (and (sequence-equal x result) result))
  #[x x 3 x x x x x x])
