(defmethod sequence:make-sequence-iterator :around ((o undoable-mixin) &key from-end start end)
  (declare (ignore from-end start end))
  (multiple-value-bind (s l f step endp elt setelt index copy) (call-next-method)
    (values s l f step endp elt
            (lambda (nv s i)
              (if (recording s)
                  (funcall setelt nv s i)
                  (without-undoing (s)
                    (push (make-instance 'setelt-record :index (funcall index s i) :value (funcall elt s i)) 
                          (record s))
                    (funcall setelt nv s i))))
            index copy)))

(defmacro with-compound-recording ((object) &body body)
  `(let ((old (recording ,object)) (or (record ,object)))
    (unwind-protect
         (progn (setf (recording ,object) nil) (setf (record ,object) nil) ,@body)
      (setf (recording ,object) old)
      (setf (record ,object) (cons (make-instance 'compound-record :records (record ,object)) or)))))

(defclass compound-record () ((records :initarg :records :reader records)))
(defmethod undo-using-record ((o undoable-mixin) (r compound-record))
  (dolist (r (records r)) (undo-using-record o r)))
(defmacro define-compound-undo-method (name arglist)
  `(define-undo-method ,name ,arglist
     (with-compound-recording (o) (call-next-method))))

(define-compound-undo-method sequence:nsubstitute (new old o &key from-end start end test test-not key count))
(define-compound-undo-method sequence:nsubstitute-if (new old o &key from-end start end key count))
(define-compound-undo-method sequence:nsubstitute-if-not (new old o &key from-end start end key count))

(defclass adjust-sequence-record ()
  ((previous :initarg :previous :reader previous) (discarded :initarg :discarded :reader discarded)))
(define-undo-method sequence:adjust-sequence (o length &rest args)
  (push (make-instance 'adjust-sequence-record :previous (length o)
                       :discarded (when (< length (length o)) (coerce (subseq o length) 'vector)))
        (record o)))
(defmethod undo-using-record ((o undoable-mixin) (r adjust-sequence-record))
  (sequence:adjust-sequence o (previous r))
  (when (discarded r) (setf (subseq o (- (length o) (length (discarded r)))) (discarded r))))
            
(define-compound-undo-method sequence:delete (item o &key from-end (start 0) end test test-not key count))
(define-compound-undo-method sequence:delete-if (item o &key from-end (start 0) end key count))
(define-compound-undo-method sequence:delete-if-not (item o &key from-end (start 0) end key count))
(define-compound-undo-method sequence:delete-duplicates (o &key from-end (start 0) end test test-not key))
(define-undo-method sequence:replace (o sequence2 &key (start1 0) end1 (start2 0) end2)
  (push (make-instance 'fill-record :start start1 :end end1 :contents (coerce (subseq o start1 end1) 'vector))
        (record o)))

(define-undo-method sequence:sort (o predicate &key key)
  (push (make-instance 'fill-record :contents (coerce o 'vector) :start 0 :end nil) (record o)))
(define-undo-method sequence:stable-sort (o predicate &key key)
  (push (make-instance 'fill-record :contents (coerce o 'vector) :start 0 :end nil) (record o)))
