(defclass undoable-mixin ()
  ((recording :initform nil :accessor recording) (record :initform nil :accessor record)))
(defclass setelt-record ()
  ((index :initarg :index :reader index) (value :initarg :value :reader value)))
(defclass fill-record ()
  ((start :initarg :start :reader start) (end :initarg :end :reader end) 
   (contents :initarg :contents :reader contents)))
(defclass nreverse-record () ())
(defmacro without-undoing ((object) &body body)
  `(let ((old (recording ,object)))
    (unwind-protect (progn (setf (recording ,object) t) ,@body)
      (setf (recording ,object) old))))
(defmacro define-undo-method (name arglist &body body)
  `(defmethod ,name :around ,(substitute '(o undoable-mixin) 'o arglist)
     (if (recording o) (call-next-method) (without-undoing (o) ,@body (call-next-method)))))
(define-undo-method (setf sequence:elt) (new-value o index)
  (push (make-instance 'setelt-record :index index :value (elt o index)) (record o)))
(define-undo-method sequence:fill (o item &key (start 0) end)
  (push (make-instance 'fill-record :start start :end end :contents (subseq (coerce o 'vector) start end))
        (record o)))
(define-undo-method sequence:nreverse (o) (push (make-instance 'nreverse-record) (record o)))
(defun undo (object)
  (undo-using-record object (car (record object))) object)
(defmethod undo-using-record ((o undoable-mixin) (r null)) (error "Nothing to undo"))
(defmethod undo-using-record :after ((o undoable-mixin) r) (pop (record o)))
(defmethod undo-using-record ((o undoable-mixin) (r setelt-record))
  (without-undoing (o) (setf (elt o (index r)) (value r))))
(defmethod undo-using-record ((o undoable-mixin) (r fill-record))
  (without-undoing (o)
    (with-accessors ((start start) (end end) (contents contents)) r (setf (subseq o start end) contents))))
(defmethod undo-using-record ((o undoable-mixin) (r nreverse-record))
  (without-undoing (o) (sequence:nreverse o)))
